<?
//
// ====================================================================
// Object to encapsulate XML file retrieval and parsing.
// Customized for use with the MapRaider.com content-
// sharing system.
//
// Version 1.0.0
//
// Dependencies:
//		You must have the PHP XML parse routines installed
// --------------------------------------------------------------------
//
// --------------------------------------------------------------------
// Code written by John Dailey and is Copyright (c)
// 2003 by John Dailey unless otherwise stated.
// Based on the JScript ASP version written by Conan Theobald
//
// Visit: www.mapraider.com
// Email: info@mapraider.com
// ====================================================================
//
// ====================================================================
// Documentation
// ====================================================================
//
//
//
// --------------------------------------------------------------------
// Quick start
// --------------------------------------------------------------------
// 
// 1). First, "Include" this file in your page by inserting the following 
// line somewhere near the top:
// 
// 	include_once("inc.mapraider.php");
// 
// 2). Call the application initialization to setup the cache.
// 
// 	Cache_Init();
// 
// 3). Now create an instance of the "XMLMapraiderClass" object:
// 
// 	$oMapraider = new XMLMapRaiderClass;
// 
// 4). Specify which XML file you want to include for content-caching. You
// can specify all files with the line:
// 
// 	$oMapraider->IncludeAll();
// 
// Or you can specify individual files with this line:
// 
// 	$oMapraider->IncludeFile("New10_UT2003");
// 	$oMapraider->IncludeFile("New10_UT");
// 
// 	. . .
// 
// 5). Next you must tell the object to cache the selected files:
// 
// 	$oMapraider->Update();
// 
// If no cache exists, the "Update" method will unconditionally
// cache all the specified XML content. If there is *already* data
// in the cache when "Update" is called, then it will only be refreshed
// If that content is older than 12 hours. (Twice daily, in other words.)
// 
// 6). Now you can start working! For example, If you wanted to find
// the newest UT2003 map, use the following code:
// 
// 	$oLatestMap = $oMapraider->FindFirst("New_UT2003");
//
// FindFirst and FindNext both return a dictionary object, which you
// can access the elements of in the traditional manner:
// 
// 	sContent .= "<b>Here is the latest UT2003 map:</b><br />" .
// 		"<i>Name:</i> " . $oLatestMap["name"] . "<br />" .
// 		"<i>Author:</i> " . $oLatestMap["author"] . "<br />" .
// 		"<i>Description:</i> " . $oLatestMap["description"]
// 
// 7). Now, tell the cache to write to disk.  Very important to make sure
// you're not rereading the data every time your script runs :)
//
//  Cache_Save();
// 
// All known content-fields are documented below.
// 
// --------------------------------------------------------------------
// function reference
// 	NOTE: Only the most-used methods have been documented.
// 	There are other methods but these are only for internal use
// 	by the object, or for debugging purposes.
// --------------------------------------------------------------------
//
// ->Set_CacheFile( $filename )
//	Allows you to modify the path and filename of the cache file.
//	The default is 'mapraider.cache' in the same directory as the calling
//	script.
//
// ->Cache_Init ()
//	Initializes the cache by reading all variables from the cache file.
//
// ->Cache_Save ()
//	Writes any changes to the cache in memory to disk.
// 
// ->IncludeAll()
// 	Takes no arguments. Tells the object to include ALL XML files
// 	for content-caching.
// 
// ->ExcludeAll()
// 	Takes no arguments. Tells the object to exclude ALL XML files
// 	from content-caching.
// 
// ->IncludeFile($sFile)
// 	Takes one argument. Specifies an XML file to include for
// 	content-caching.
// 
// ->ExcludeFile($sFile)
// 	Takes one argument. Specifies an XML file to exclude from
// 	content-caching.
// 
// --------------------------------------------------------------------
// 
// Currently, there are only 27 possible values for the "$sFile" argument
// taken by "IncludeFile" and "ExcludeFile":
// 
//		'New10_MOHAA'			= The 10 newest Medal of Honor: Allied Assault maps.
//		'New10_Q3A'			= The 10 newest Quake III: Arena maps.
//		'New10_U2'			= The 10 newest Unreal II: The Awakening maps.
//		'New10_Unreal'			= The 10 newest Unreal maps.
//		'New10_UT2003'			= The 10 newest Unreal Tournament 2003 maps.
//		'New10_UT'			= The 10 newest Unreal Tournament maps.
//
//		'New10_UT_DM'			= The 10 newest DeathMatch Unreal Tournament maps.
//		'New10_UT_CTF'			= The 10 newest Capture The XMLParseFlag Unreal Tournament maps.
//		'New10_UT_AS'			= The 10 newest Assault Unreal Tournament maps.
//		'New10_UT_GIANT'		= The 10 newest Giant-sized Unreal Tournament maps.
//		'New10_UT2003_DM'		= The 10 newest DeathMatch Unreal Tournament 2003 maps.
//		'New10_UT2003_CTF'		= The 10 newest Capture The XMLParseFlag Unreal Tournament 2003 maps.
//		'New10_UT2003_GIANT'	= The 10 newest Giant-sized Unreal Tournament 2003 maps.
//
//		'New25'				= The 25 newest maps.
//
//		'Top10_MOHAA'			= Top 10 scoring Medal of Honor: Allied Assault maps.
//		'Top10_Q3A'			= Top 10 scoring Quake III: Arena maps.
//		'Top10_U2'			= Top 10 scoring Unreal II: The Awakening maps.
//		'Top10_Unreal'			= Top 10 scoring Unreal maps.
//		'Top10_UT2003'			= Top 10 scoring Unreal Tournament 2003 maps.
//		'Top10_UT'			= Top 10 scoring Unreal Tournament maps.
//		
//		'Top10_UT_DM'			= Top 10 scoring DeathMatch Unreal Tournament maps.
//		'Top10_UT_CTF'			= Top 10 scoring Capture The XMLParseFlag Unreal Tournament maps.
//		'Top10_UT_AS'			= Top 10 scoring Assault Unreal Tournament maps.
//		'Top10_UT_GIANT'		= Top 10 scoring Giant-sized Unreal Tournament maps.
//		'Top10_UT2003_DM'		= Top 10 scoring DeathMatch Unreal Tournament 2003 maps.
//		'Top10_UT2003_CTF'		= Top 10 scoring Capture The XMLParseFlag Unreal Tournament 2003 maps.
//		'Top10_UT2003_GIANT'	= Top 10 scoring Giant-sized Unreal Tournament 2003 maps.
//
// NOTE: These are also the only possible values for the "$sName" argument
// taken by "FindFirst", documented below.
// 
// --------------------------------------------------------------------
// 
// ->Update()
// 	Takes no arguments. This is the main content-caching method, and
// 	should be called *after* deciding which XML files to cache using the
// 	four methods documented above.
// 	
// 	"Update" will only re-cache content If there is *no* cache, or If the contents
// 	of the cache is older than 12 hours. (ie, refreshed twice daily.)
// 
// ->FindFirst($sName)
// 	Takes one argument and returns an expando collection. Use this method
// 	to find the first item (map) in the specified list. You can then access the
// 	returned data via expando properties which represent the XML content-fields:
// 
// 	Here is the list of currently known content-fields (04/23/2003):
// 
// 		title		= Map title (name + version)
// 		summary		= Map summary (name + author + rating)
// 		url			= Download URL
// 		link			= Download URL
// 		image		= Image URL
// 		screenshot	= Image URL
// 		mapname		= Map name
// 		version		= Map version
// 		author		= Map author
// 		authorprofile	= URL to authors profile
// 		description	= Full map description
// 		dateadded		= The date the map was added to MapRaider
// 		score		= The map score
// 		ratings		= Total number of map ratings
// 
// 	Example code:
// 
//		// Initialization
// 
// 		include_once("inc.mapraider.php");
//
//		Cache_Init();
// 
// 		$oMapraider = new XMLMapRaiderClass;
// 
//
//		$oMapraider->IncludeFile("Top10_UT2003");
//		$oMapraider->Update();
// 
//		// Get the Title and Description of the latest UT2003 map
// 
// 		$oLatestMap = $oMapraider->FindFirst("New10_UT2003");
// 		$sMapTitle = $oLatestMap["title"];
// 		$sMapDescription = $oLatestMap["description"];
// 
// 		. . .
// 
// ->FindNext()
// 	Takes no arguments and returns an expando collection. Use this method
// 	to find the *next* item (map) in the list as previously specified by the
// 	"FindFirst" method. Returns null If the end of the list has been reached.
// 
// 	Example code:
// 
//		// Initialization
// 
// 		include_once("inc.mapraider.php");
//
//		Cache_Init();
// 
// 		$oMapraider = new XMLMapRaiderClass;
// 
//
//		$oMapraider->IncludeFile("Top10_UT2003");
//		$oMapraider->Update();
// 
//		// Display the Top-10 highest rated Quake III: Arena maps:
// 
// 		$sHTML = ""
// 		$oMap = $oMapraider->FindFirst("Top10_Q3A");
// 
//		while ( !is_null($oMap) ) {
//		
//			$sHTML = $sHTML .
//				"<p style=\"clear: both;\">" .
//					"<img src=\"" . $oMap["screenshot"] . "\" align=\"left\" />" .
//					"<b>Name:</b> " . $oMap["title"] . "<br />" .
//					"<b>Score:</b> " . $oMap["score"] . "<br />" .
//					"<b>Description:</b> " . $oMap["description"] .
//				"</p>";
//				
//			$oMap = $oMapraider->FindNext();
//		
//		}
// 
// 		echo $sHTML;
//
//		$oMapraider = null;
//
//		Cache_Save();
// 
//



// ====================================================================
// Useful functions
// ====================================================================
	
	
	
	// ====================================================================
	// XML Parsing Methods used by the class.  These routines and variables
	// are use by events triggered from the PHP XML parser and must reside
	// in the global program scope outside of the class.  You should have
	// no need to modify these.
	// ====================================================================
	
	// set up some variables for use by the parser
	$XMLParsecurrentTag = "";
	$XMLParseFlag = "";
	$XMLParseCount = 0;
	
	// this is an associative array of channel data
	$XMLParseChannel = array();
	
	// this is an array of arrays, with each array element representing an <item> 
	// each outer array element is itself an associative array
	$XMLParseItems = array();
	
	
	// opening tag handler
	function elementBegin ( $parser, $name, $attributes ) {
	
		global $XMLParsecurrentTag, $XMLParseFlag;
		
		$XMLParsecurrentTag = $name;
		
		// set XMLParseFlag if entering <channel> or <item> block
		if ($name == "ITEM") {
		
			$XMLParseFlag = 1;
			
		} else if ($name == "CHANNEL") {
		
			$XMLParseFlag = 2;
			
		}
		
	}
	
	
	// closing tag handler       
	function elementEnd ( $parser, $name ) {
	
		global $XMLParsecurrentTag, $XMLParseFlag, $XMLParseCount;
		
		$XMLParsecurrentTag = "";
	
		// set XMLParseFlag if exiting <channel> or <item> block
		if ($name == "ITEM") {
		
			$XMLParseCount++;
			$XMLParseFlag = 0;
			
		} else if ($name == "CHANNEL") {
		
			$XMLParseFlag = 0;
		}
		
	}
	
	
	// character data handler
	function characterData ( $parser, $data ) {
	
		global $XMLParsecurrentTag, $XMLParseFlag, $XMLParseItems, $XMLParseCount, $XMLParseChannel;
		
		$data = trim(htmlspecialchars($data));
		
		// add data to $XMLParseChannels[] or $XMLParseItems[] array
		if ($XMLParseFlag == 1) {
		
			$XMLParseItems[$XMLParseCount][strtolower($XMLParsecurrentTag)] .= $data;
			
		} else if ($XMLParseFlag == 2) {
		
			$XMLParseChannel[strtolower($XMLParsecurrentTag)] .= $data;
		}
	
	}
	

	// ====================================================================
	// Searches for a value within the specified 2D array.
	// Returns the index (base "0") of the element If found, null If not
	// oElement refers to the element you want to compare oItem to.
	// ====================================================================
	
	function array_getIndex ( $aArrayData, $oItem, $iElement ) {
	
		if ( is_array( $aArrayData ) ) {

			$iOut = null;
			$iUpper = count($aArrayData);
			
			for($i = 0; $i < $iUpper; $i++) {
				
				if ( $aArrayData[$i][$iElement] == $oItem ) {
					$iOut = $i;
					break;
				}
				
			}
				
		} else {
		
			$iOut = null;
		
		}
		
		return $iOut;
		
	}
	

// ====================================================================
// Main object
// ====================================================================

Class XMLMapraiderClass {


	// ====================================================================
	// Variables
	// ====================================================================
	
	var $XMLFiles = array();
	var $oData = null;
	var $Collection = "";
	var $Position = 0;
	var $mapraidercachefile = "";


	// ====================================================================
	// Class initialization
	// ====================================================================
	
	function XMLMapraiderClass () {
	
		global $XMLFiles, $Collection, $Position, $mapraidercachefile;
		
		$Collection = "";
		$Position = 0;
		$mapraidercachefile = "mapraider.cache";
		
		$XMLFiles = array (
		
			array( false, "New10_MOHAA",		"http://www.mapraider.com/sharing/xml/new10-mohaa.xml" ),
			array( false, "New10_Q3A",		"http://www.mapraider.com/sharing/xml/new10-q3a.xml" ),
			array( false, "New10_U2",		"http://www.mapraider.com/sharing/xml/new10-u2.xml" ),
			array( false, "New10_Unreal",		"http://www.mapraider.com/sharing/xml/new10-unreal.xml" ),
			array( false, "New10_UT2003",		"http://www.mapraider.com/sharing/xml/new10-ut2k3.xml" ),
			array( false, "New10_UT",		"http://www.mapraider.com/sharing/xml/new10-ut.xml" ),
			
			array( false, "New25",			"http://www.mapraider.com/sharing/xml/new25.xml" ),
		
			array( false, "New10_UT_DM",		"http://www.mapraider.com/sharing/xml/new10-ut_dm.xml" ),
			array( false, "New10_UT_CTF",		"http://www.mapraider.com/sharing/xml/new10-ut_ctf.xml" ),
			array( false, "New10_UT_AS",		"http://www.mapraider.com/sharing/xml/new10-ut_as.xml" ),
			array( false, "New10_UT_GIANT",	"http://www.mapraider.com/sharing/xml/new10-ut_giant.xml" ),
			array( false, "New10_UT2003_DM",	"http://www.mapraider.com/sharing/xml/new10-ut2k3_dm.xml" ),
			array( false, "New10_UT2003_CTF",	"http://www.mapraider.com/sharing/xml/new10-ut2k3_ctf.xml" ),
			array( false, "New10_UT2003_GIANT","http://www.mapraider.com/sharing/xml/new10-ut2k3_giant.xml" ),

			array( false, "Top10_MOHAA",		"http://www.mapraider.com/sharing/xml/top10-mohaa.xml" ),
			array( false, "Top10_Q3A",		"http://www.mapraider.com/sharing/xml/top10-q3a.xml" ),
			array( false, "Top10_U2",		"http://www.mapraider.com/sharing/xml/top10-u2.xml" ),
			array( false, "Top10_Unreal",		"http://www.mapraider.com/sharing/xml/top10-unreal.xml" ),
			array( false, "Top10_UT2003",		"http://www.mapraider.com/sharing/xml/top10-ut2k3.xml" ),
			array( false, "Top10_UT",		"http://www.mapraider.com/sharing/xml/top10-ut.xml" ),
		
			array( false, "Top10_UT_DM",		"http://www.mapraider.com/sharing/xml/top10-ut_dm.xml" ),
			array( false, "Top10_UT_CTF",		"http://www.mapraider.com/sharing/xml/top10-ut_ctf.xml" ),
			array( false, "Top10_UT_AS",		"http://www.mapraider.com/sharing/xml/top10-ut_as.xml" ),
			array( false, "Top10_UT_GIANT",	"http://www.mapraider.com/sharing/xml/top10-ut_giant.xml" ),
			array( false, "Top10_UT2003_DM",	"http://www.mapraider.com/sharing/xml/top10-ut2k3_dm.xml" ),
			array( false, "Top10_UT2003_CTF",	"http://www.mapraider.com/sharing/xml/top10-ut2k3_ctf.xml" ),
			array( false, "Top10_UT2003_GIANT", "http://www.mapraider.com/sharing/xml/top10-ut2k3_giant.xml" )
			
		);
		
	}

	
	// ====================================================================
	// Methods
	// ====================================================================


	// ====================================================================
	// The following three functions simulate ASP application variables by
	// storing variable data to a text file on the server's hard drive.
	// 
	// !!!!!! Note that use of these routines in a multi-user environment 
	// WILL cause problems if both users modify the application variables!
	//
	// You MUST call Cache_Init() at the beginning of each PHP
	// file and Cache_Save() at the end for this to work properly.
	// ====================================================================
	
	
	// Define a new cache file location and name
	function Set_CacheFile ( $filename ) {
	
		global $mapraidercachefile;
		
		$mapraidercachefile = $filename;
		
	}
	
	
	// Read in our application variables from the data file
	function Cache_Init () {
	
		global $_APP, $mapraidercachefile;
	
		// if data file exists, load application variables
		if (file_exists($mapraidercachefile) && filesize($mapraidercachefile) > 0) {
		
			// read data file
			$file = fopen($mapraidercachefile, "r");
			
			if ($file) {
			
				$data = fread( $file, filesize($mapraidercachefile) );
				fclose($file);
				
			}
	
			// build application variables from data file
			$_APP = unserialize($data);
		}
		
	}
	

	// Write our application variables from the data file
	function Cache_Save () {
	
		global $_APP, $mapraidercachefile;
	
		// write application data to file
		$data = serialize($_APP);
		@chmod($mapraidercachefile,0777);
		$file = fopen($mapraidercachefile, "w");
		
		if ($file) {
		
			fwrite($file, $data);
			fclose($file);
			@chmod($mapraidercachefile,0644);
			
		}
		
	}
	

	// Include an XML for caching
	function IncludeFile ( $sFile ) {
	
		global $XMLFiles;
		
		$iIndex = array_getIndex($XMLFiles, $sFile, 1);
		
		if( !is_null($iIndex) ) {
		
			$XMLFiles[$iIndex][0] = true;
			
		}
		
	}


	// Exclude an XML from caching
	function ExcludeFile ( $sFile ) {
	
		global $XMLFiles;
	
		$iIndex = array_getIndex($XMLFiles, $sFile, 1);
		
		if( !is_null($iIndex) ) {
		
			$XMLFiles[$iIndex][0] = false;
			
		}
		
	}


	// Include *all* XML files for content caching
	function IncludeAll () {
	
		global $XMLFiles;
		
		for ($i = 0; $i < count($XMLFiles); $i++) {
			$XMLFiles[$i][0] = true;
		}
		
	}


	// Exclude *all* XML files from content caching
	function ExcludeAll () {
	
		global $XMLFiles;
		
		for ($i = 0; $i < count($XMLFiles); $i++) {
			$XMLFiles[$i][0] = false;
		}
		
	}
	
	
	// Get the first map from the specified collection
	function FindFirst ( $sName ) {
	
		global $Collection, $Position, $oData;
		
		$aTemp = array();
		$Collection = $sName;
		$Position = 0;
		
		if(is_array($oData[$Collection])) {
		
			$aTemp = $oData[$Collection];
			return $aTemp[1][$Position];
			
		} else {
		
			return null;
			
		}

	}


	// Get the next map in the list... (be sure to call FindFirst at least once before calling this function)
	function FindNext () {
	
		global $Collection, $Position, $oData;
	
		$iUpper = 0;
		$aTemp = array();
		
		$aTemp = $oData[$Collection];
		$iUpper = count($aTemp[1]);
		
		if($iUpper > 0) {

			$Position++;
		
			if($Position <= $iUpper) {
			
				return $aTemp[1][$Position];
				
			} else {
			
				return null;
				
			}
			
		} else {
		
			return null;
			
		}
		
	}


	// Set application cache
	function SetCache ( $sName, $sValue ) {
	
		global $_APP;
		$_APP[$sName] = $sValue;
		
	}


	// Retrieve application cache
	function GetCache ( $sName ) {
	
		global $_APP;
		return $_APP[$sName];
		
	}
	
	
	// clear all of our cache data
	// you should not use this method unless you absolutely
	// need to erase the application variables.  Do not
	// include it in your production code since it effectively
	// breaks the caching mechanism.
	function ClearCache () {
	
		global $_APP, $mapraidercachefile;
		
		unlink($mapraidercachefile);
	
		// null out our local application variable too.
		$_APP = null;
		
	}


	// This function unconditionally re-caches all MapRaider content.
	// The content is stored in application scope.
	function PreCache () {
	
		global $XMLFiles, $XMLParseItems, $XMLParseChannel, $XMLParseFlag, $XMLParseCount;

		// Variables...
		$oNode = null;
		$oNodeList = null;
		$oValues = null;
		
		$oParamaters = array();
		
		$iMapCount = 0;
		
		$sTitle = "";
		$sPageTitle = "";
		$sNodeName = "";
		$sNodeValue = "";
		
		$i = 0;
		$j = 0;
		$k = 0;

		// Loop through all XMLFiles...
		for($i = 0; $i < count($XMLFiles); $i++) {

			// ...but only those that have been "activated"
			if ( $XMLFiles[$i][0] ) {
			
				$bResult = true;

				$sTitle = $XMLFiles[$i][1];
				
				// initialize our parse variables
				// set up some variables for use by the parser
				$XMLParsecurrentTag = "";
				$XMLParseFlag = "";
				$XMLParseCount = 0;
				
				// this is an associative array of channel data
				$XMLParseChannel = array();
				
				// this is an array of arrays, with each array element representing an <item> 
				// each outer array element is itself an associative array
				$XMLParseItems = array();
				
				// create parser
				$oXML = xml_parser_create();
				
				// set element handler
				xml_set_element_handler($oXML, "elementBegin", "elementEnd");
				xml_set_character_data_handler($oXML, "characterData");
				xml_parser_set_option($oXML, XML_OPTION_CASE_FOLDING, TRUE);
				xml_parser_set_option($oXML, XML_OPTION_SKIP_WHITE, TRUE);
				
				// read XML file
				if ( !( $fp = fopen($XMLFiles[$i][2], "r") ) ) {
					$bResult = false;
					 echo "Could not read " . $XMLFiles[$i][2];
				}
				
				// parse data in 4096byte chunks
				while ( $xml = fread($fp, 4096) ) {
				
					if ( !xml_parse( $oXML, $xml, feof($fp) ) ) {
						$bResult = false;
						echo "XML parser error: " . xml_error_string(xml_get_error_code($oXML));
					}
					
				}
				
				// destroy parser
				xml_parser_free($oXML);

				if ($bResult) {

					// Select node pertaining to the file description
					$sPagetitle = $XMLParseChannel["description"];

					// Cache map-list description
					$this->SetCache($sTitle . "_Description", $sPagetitle);

					// Cache map-list length

					$iMapCount = count($XMLParseItems);
					$this->SetCache($sTitle . "_MapCount", $iMapCount);
					
					// Cache all map data
					for($k = 0; $k < $iMapCount; $k++) {

						// Loop through the child-nodes, populating the 
						// appropriate variables with their respective values...

						for($j = 0; $j < count($XMLParseItems[$k]); $j++) {
						
							$sNodeName = array_keys($XMLParseItems[$k]);
							$sNodeName = $sNodeName[$j];
							$sNodeValue = $XMLParseItems[$k][$sNodeName];
							
							// Add node-name to parameter list If it does not exist
							if ( !in_array($sNodeName, $oParamaters) && $sNodeName) {
								$oParamaters[] = $sNodeName;
							}

							// Add to cache
							if ($sNodeName) {
								$this->SetCache($sTitle . "_Map_" . $k . "_" . $sNodeName, $sNodeValue);
							}

						}

					}

					// Store node-names
					$this->SetCache( $sTitle . "_Paramaters", join($oParamaters, ",") );

				}

			}

		}

		// Clean up
		$oXML = null;

	}
	

	// This function retrieves the cache into a local array
	function Refresh () {
	
		global $XMLFiles, $oData;

		$sTitle = "";
		$sDescription = "";
		$aParamaters = "";
		
		$iMapCount = 0;
		
		$aMapArray = array();

		// Loop through all XMLFiles...
		for($i = 0; $i < count($XMLFiles) - 1; $i++) {

			// ...but only those that have been "activated"
			if ( $XMLFiles[$i][0] ) {

				// Get title for this XML file
				$sTitle = $XMLFiles[$i][1];

				// Get the page-description from the cache
				$sDescription = $this->GetCache($sTitle . "_Description");

				// Get the list of parameter-names from the cache and split into an array
				$aParamaters = explode(",", $this->GetCache($sTitle . "_Paramaters"));

				// Get the number of maps that are in cache
				if (strlen( $this->GetCache($sTitle . "_MapCount") ) > 0) {
					$iMapCount = 0 + $this->GetCache($sTitle . "_MapCount");
				}
				
				// Get rid of our old array
				$aMapArray = array();

				// Retrieve all cached map-details for this XML file
				for($j = 0; $j < $iMapCount; $j++) {

					// Loop through all paramaters, getting the node-name and
					// node-values for each
					
					for($k = 0; $k < count($aParamaters); $k++) {
						
						$aMapArray[$j][ $aParamaters[$k] ] = $this->GetCache( $sTitle . 
							"_Map_" . $j . "_" . $aParamaters[$k] );
					}

				}
				
				// Assign our array of objects to our main array
				$oData[$sTitle] = array( $sDescription, $aMapArray );

			}

		}

	}
	

	// Cache data If it does not exist, and refresh it If it is out-of-date
	function Update () {
	
		$dToday = time();
		$sDate = null;

		// Get date of last content-refresh from application scope
		$sDate = $this->GetCache("MapRaider_LastUpdate");

		// No date? PreCache...
		if ( !$sDate ) {

			$this->PreCache();

			// Update last content-refresh date
			$this->SetCache("MapRaider_LastUpdate", $dToday);

		// If date is found, compare it against todays date...
		} else {

			// Refresh content every 12 hours
			if (( (($dToday - $sDate) / 60) / 60) > 11) {

				$this->PreCache();

				// Update last content-refresh date
				$this->SetCache("MapRaider_LastUpdate", $dToday);

			}

		}

		// poulate local variables with the cache
		$this->Refresh();

	}
	

}

?>